@@ -145,16 +145,16 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust)
 			printf("  Creation Time : %.24s\n", ctime(&atime));
 			c=map_num(pers, super.level);
 			printf("     Raid Level : %s\n", c?c:"-unknown-");
-			printf("    Device Size : %d%s\n", super.size, human_size((long long)super.size<<10));
+			if (super.level <= 0)
+				printf("    Device Size : %u%s\n", super.size, human_size((long long)super.size<<10));
 			printf("   Raid Devices : %d\n", super.raid_disks);
 			printf("  Total Devices : %d\n", super.nr_disks);
 			printf("Preferred Minor : %d\n", super.md_minor);
 			printf("\n");
 			atime = super.utime;
 			printf("    Update Time : %.24s\n", ctime(&atime));
-			printf("          State : %s, %serrors\n",
-			       (super.state&(1<<MD_SB_CLEAN))?"clean":"dirty",
-			       (super.state&(1<<MD_SB_ERRORS))?"":"no-");
+			printf("          State : %s\n",
+			       (super.state&(1<<MD_SB_CLEAN))?"clean":"dirty");
 			printf(" Active Devices : %d\n", super.active_disks);
 			printf("Working Devices : %d\n", super.working_disks);
 			printf(" Failed Devices : %d\n", super.failed_disks);
@@ -175,11 +175,20 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust)
 				printf (" --- adjusting superblock for 2.2/sparc compatability ---\n");
 			}
 			printf("         Events : %d.%d\n", super.events_hi, super.events_lo);
+			if (super.events_hi == super.cp_events_hi &&
+			    super.events_lo == super.cp_events_lo &&
+			    super.recovery_cp > 0 &&
+			    (super.state & (1<<MD_SB_CLEAN)) == 0 )
+				printf("Sync checkpoint : %d KB (%d%%)\n", super.recovery_cp/2, super.recovery_cp/(super.size/100*2));
 			printf("\n");
 			if (super.level == 5) {
 				c = map_num(r5layout, super.layout);
 				printf("         Layout : %s\n", c?c:"-unknown-");
 			}
+			if (super.level == 10)
+				printf("         Layout : near=%d, far=%d\n",
+				       super.layout&255, (super.layout>>8) & 255);
+
 			switch(super.level) {
 			case 0:
 			case 4:
@@ -210,6 +219,7 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust)
 				if ((dv=map_dev(dp->major, dp->minor)))
 					printf("   %s", dv);
 				printf("\n");
+				if (d == -1) printf("\n");
 			}
 		}
 		if (SparcAdjust == 2) {
@@ -236,8 +246,9 @@ int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust)
 			char sep='=';
 			char *c=map_num(pers, ap->super.level);
 			char *d;
-			printf("ARRAY /dev/md%d level=%s num-devices=%d UUID=",
-			       ap->super.md_minor, c?c:"-unknown-", ap->super.raid_disks);
+			printf("ARRAY %s level=%s num-devices=%d UUID=",
+			       get_md_name(ap->super.md_minor),
+			       c?c:"-unknown-", ap->super.raid_disks);
 			if (spares) printf(" spares=%d", spares);
 			if (ap->super.minor_version >= 90)
 				printf("%08x:%08x:%08x:%08x", ap->super.set_uuid0, ap->super.set_uuid1,
